@@ -42,20 +42,21 @@
 //! ```rust,no_run
 //! use openstack;
 //!
-//! let auth = openstack::auth::Identity::new("https://my.cloud.com/identity").unwrap()
-//!     .with_user("admin", "pa$$w0rd", "My Domain")
-//!     .with_project_scope("project1", "My Domain")
-//!     .create().expect("Failed to authenticate");
+//! let auth = openstack::auth::Password::new(
+//!         "https://my.cloud.com/identity",
+//!         "admin", "pa$$w0rd", "domain.com")
+//!     .expect("Invalid authentication URL")
+//!     .with_project_scope("project1", "domain.com");
 //! let os = openstack::Cloud::new(auth);
 //! ```
 //!
-//! Creating an authentication method from environment variables:
+//! Creating a session and a cloud from environment variables:
 //!
 //! ```rust,no_run
 //! use openstack;
 //!
-//! let auth = openstack::auth::from_env().expect("Failed to authenticate");
-//! let os = openstack::Cloud::new(auth);
+//! let session = openstack::auth::from_env().expect("Failed to authenticate");
+//! let os = openstack::Cloud::from(session);
 //! ```
 //!
 //! Creating a dummy authentication method for use against clouds that do not
@@ -74,49 +75,48 @@
 //! * Only Identity API v3 is supported and planned for support.
 
 mod base;
+mod config;
 mod identity;
 mod simple;
 
 pub use self::base::{AuthMethod, BoxedClone};
+pub use self::config::from_config;
 pub use self::simple::NoAuth;
-pub use self::identity::{Identity, PasswordAuth};
+pub use self::identity::{Identity, Password};
 
 use std::env;
 
 use super::{Error, ErrorKind, Result};
+use super::session::Session;
 
-const MISSING_ENV_VARS: &'static str =
+const MISSING_ENV_VARS: &str =
     "Not all required environment variables were provided";
-const INVALID_ENV_AUTH_URL: &'static str =
-    "Malformed authentication URL provided in the environment";
 
 #[inline]
 fn _get_env(name: &str) -> Result<String> {
     env::var(name).map_err(|_| {
-        Error::new(ErrorKind::InvalidInput,
-                                MISSING_ENV_VARS)
+        Error::new(ErrorKind::InvalidInput, MISSING_ENV_VARS)
     })
 }
 
 
-/// Create an authentication method from environment variables.
-pub fn from_env() -> Result<PasswordAuth> {
-    let auth_url = _get_env("OS_AUTH_URL")?;
-    let id = Identity::new(&auth_url).map_err(|_| {
-        Error::new(ErrorKind::InvalidInput,
-                                INVALID_ENV_AUTH_URL)
-    })?;
+/// Create a `Session` from environment variables.
+pub fn from_env() -> Result<Session> {
+    if let Ok(cloud_name) = env::var("OS_CLOUD") {
+        from_config(cloud_name)
+    } else {
+        let auth_url = _get_env("OS_AUTH_URL")?;
+        let user_name = _get_env("OS_USERNAME")?;
+        let password = _get_env("OS_PASSWORD")?;
+        let user_domain = env::var("OS_USER_DOMAIN_NAME")
+            .unwrap_or(String::from("Default"));
 
-    let user_name = _get_env("OS_USERNAME")?;
-    let password = _get_env("OS_PASSWORD")?;
-    let project_name = _get_env("OS_PROJECT_NAME")?;
+        let id = Password::new(&auth_url, user_name, password, user_domain)?;
 
-    let user_domain = env::var("OS_USER_DOMAIN_NAME")
-        .unwrap_or(String::from("Default"));
-    let project_domain = env::var("OS_PROJECT_DOMAIN_NAME")
-        .unwrap_or(String::from("Default"));
+        let project_name = _get_env("OS_PROJECT_NAME")?;
+        let project_domain = env::var("OS_PROJECT_DOMAIN_NAME")
+            .unwrap_or(String::from("Default"));
 
-    id.with_user(user_name, password, user_domain)
-        .with_project_scope(project_name, project_domain)
-        .create()
+        Ok(Session::new(id.with_project_scope(project_name, project_domain)))
+    }
 }
