@@ -62,7 +62,8 @@ pub struct Version {
     #[serde(deserialize_with = "deser_version")]
     pub id: ApiVersion,
     pub links: Vec<Link>,
-    pub status: String,
+    #[serde(deserialize_with = "empty_as_none", default)]
+    pub status: Option<String>,
     #[serde(deserialize_with = "empty_as_none", default)]
     pub version: Option<ApiVersion>,
     #[serde(deserialize_with = "empty_as_none", default)]
@@ -72,8 +73,8 @@ pub struct Version {
 #[derive(Clone, Debug, Deserialize)]
 #[serde(untagged)]
 pub enum Root {
-    Versions { versions: Vec<Version> },
-    Version { version: Version }
+    MultipleVersions { versions: Vec<Version> },
+    OneVersion { version: Version },
 }
 
 /// Information about API endpoint.
@@ -90,6 +91,15 @@ pub struct ServiceInfo {
 }
 
 impl Version {
+    pub fn is_stable(&self) -> bool {
+        if let Some(ref status) = self.status {
+            let upper = status.to_uppercase();
+            upper == "STABLE" || upper == "CURRENT" || upper == "SUPPORTED"
+        } else {
+            true
+        }
+    }
+
     pub fn into_service_info(self) -> Result<ServiceInfo> {
         let endpoint = match self.links.into_iter().find(|x| &x.rel == "self") {
             Some(link) => link.href,
@@ -109,6 +119,36 @@ impl Version {
     }
 }
 
+impl Root {
+    /// Extract `ServiceInfo` from a version discovery root.
+    pub fn into_service_info<Srv: ServiceType>(self) -> Result<ServiceInfo> {
+        match self {
+            Root::OneVersion { version: ver } => {
+                if Srv::major_version_supported(ver.id) {
+                    if ! ver.is_stable() {
+                        warn!("Using version {:?} of {} API that is not marked as stable",
+                              ver, Srv::catalog_type());
+                    }
+
+                    ver.into_service_info()
+                } else {
+                    Err(Error::new(ErrorKind::EndpointNotFound,
+                                   "Major version not supported"))
+                }
+            },
+            Root::MultipleVersions { versions: mut vers } => {
+                vers.sort_unstable_by_key(|x| x.id);
+                match vers.into_iter().rfind(|x| {
+                    x.is_stable() && Srv::major_version_supported(x.id)
+                }) {
+                    Some(ver) => ver.into_service_info(),
+                    None => Err(Error::new_endpoint_not_found(Srv::catalog_type()))
+                }
+            }
+        }
+    }
+}
+
 impl ServiceInfo {
     /// Generic code to extract a `ServiceInfo` from a URL.
     pub fn fetch<Srv: ServiceType>(endpoint: Url, auth: &AuthMethod) -> Result<ServiceInfo> {
@@ -122,28 +162,11 @@ impl ServiceInfo {
         let result = auth.request(Method::GET, endpoint.clone())?.send_checked();
         match result {
             Ok(mut resp) => {
-                let mut info = match resp.json()? {
-                    Root::Version { version: ver } => {
-                        trace!("The major version for {} service from {}: {:?}",
-                               service_type, endpoint, ver);
-                        if Srv::major_version_supported(ver.id) {
-                            ver.into_service_info()
-                        } else {
-                            Err(Error::new(ErrorKind::EndpointNotFound,
-                                           "Major version not supported"))
-                        }
-                    },
-                    Root::Versions { versions: mut vers } => {
-                        vers.sort_unstable_by_key(|x| x.id);
-                        trace!("Available major versions for {} service from {}: {:?}",
-                               service_type, endpoint, vers);
-                        match vers.into_iter()
-                                .rfind(|x| Srv::major_version_supported(x.id)) {
-                            Some(ver) => ver.into_service_info(),
-                            None => Err(Error::new_endpoint_not_found(service_type))
-                        }
-                    }
-                }?;
+                let root = resp.json::<Root>()?;
+                trace!("Available major versions for {} service from {}: {:?}",
+                       service_type, endpoint, root);
+
+                let mut info = root.into_service_info::<Srv>()?;
 
                 // Older Nova returns insecure URLs even for secure protocol.
                 if secure {
@@ -249,3 +272,112 @@ pub fn ser_opt_mac<S>(value: &Option<MacAddress>, serializer: S)
         where S: Serializer {
     value.map(|m| m.to_hex_string()).serialize(serializer)
 }
+
+
+#[cfg(test)]
+mod test {
+    use reqwest::Url;
+
+    use super::super::super::ErrorKind;
+    use super::super::ApiVersion;
+    use super::{Link, Version};
+
+    #[test]
+    fn test_version_current_is_stable() {
+        let stable = Version {
+            id: ApiVersion(2, 0),
+            links: Vec::new(),
+            status: Some("CURRENT".to_string()),
+            version: None,
+            min_version: None,
+        };
+        assert!(stable.is_stable());
+    }
+
+    #[test]
+    fn test_version_stable_is_stable() {
+        let stable = Version {
+            id: ApiVersion(2, 0),
+            links: Vec::new(),
+            status: Some("Stable".to_string()),
+            version: None,
+            min_version: None,
+        };
+        assert!(stable.is_stable());
+    }
+
+    #[test]
+    fn test_version_supported_is_stable() {
+        let stable = Version {
+            id: ApiVersion(2, 0),
+            links: Vec::new(),
+            status: Some("supported".to_string()),
+            version: None,
+            min_version: None,
+        };
+        assert!(stable.is_stable());
+    }
+
+    #[test]
+    fn test_version_no_status_is_stable() {
+        let stable = Version {
+            id: ApiVersion(2, 0),
+            links: Vec::new(),
+            status: None,
+            version: None,
+            min_version: None,
+        };
+        assert!(stable.is_stable());
+    }
+
+    #[test]
+    fn test_version_deprecated_is_not_stable() {
+        let unstable = Version {
+            id: ApiVersion(2, 0),
+            links: Vec::new(),
+            status: Some("DEPRECATED".to_string()),
+            version: None,
+            min_version: None,
+        };
+        assert!(!unstable.is_stable());
+    }
+
+    #[test]
+    fn test_version_into_service_info() {
+        let url = Url::parse("https://example.com/v2").unwrap();
+        let ver = Version {
+            id: ApiVersion(2, 0),
+            links: vec![Link{
+                href: Url::parse("https://example.com/docs").unwrap(),
+                rel: "other".to_string(),
+            }, Link {
+                href: url.clone(),
+                rel: "self".to_string(),
+            }],
+            status: None,
+            version: Some(ApiVersion(2, 2)),
+            min_version: None,
+        };
+        let info = ver.into_service_info().unwrap();
+        assert_eq!(info.root_url, url);
+        assert_eq!(info.major_version, ApiVersion(2, 0));
+        assert_eq!(info.current_version, Some(ApiVersion(2, 2)));
+        assert_eq!(info.minimum_version, None);
+    }
+
+    #[test]
+    fn test_version_into_service_info_no_self_link() {
+        let ver = Version {
+            id: ApiVersion(2, 0),
+            links: vec![Link{
+                href: Url::parse("https://example.com/docs").unwrap(),
+                rel: "other".to_string(),
+            }],
+            status: None,
+            version: Some(ApiVersion(2, 2)),
+            min_version: None,
+        };
+        let err = ver.into_service_info().err().unwrap();
+        assert_eq!(err.kind(), ErrorKind::InvalidResponse);
+    }
+}
