@@ -15,24 +15,25 @@
 //! Server management via Compute API.
 
 use std::collections::HashMap;
-use std::fmt::Debug;
 use std::net::{IpAddr, Ipv4Addr, Ipv6Addr};
 use std::rc::Rc;
 use std::time::Duration;
 
 use chrono::{DateTime, FixedOffset};
 use fallible_iterator::{IntoFallibleIterator, FallibleIterator};
-use serde::Serialize;
 use waiter::{Waiter, WaiterCurrentState};
 
 use super::super::{Error, ErrorKind, Result, Sort};
-use super::super::common::{self, DeletionWaiter, FlavorRef, ImageRef, KeyPairRef,
-                           ListResources, NetworkRef, PortRef, ProjectRef,
-                           Refresh, ResourceId, ResourceIterator, UserRef};
+use super::super::common::{self, DeletionWaiter, FlavorRef, ImageRef,
+                           IntoVerified, KeyPairRef, NetworkRef,
+                           PortRef, ProjectRef, Refresh, ResourceQuery,
+                           ResourceIterator, UserRef};
+#[cfg(feature = "image")]
+use super::super::image::Image;
 use super::super::session::Session;
 use super::super::utils::Query;
 use super::base::V2API;
-use super::protocol;
+use super::{protocol, KeyPair};
 
 
 /// A query to server list.
@@ -43,11 +44,20 @@ pub struct ServerQuery {
     can_paginate: bool,
 }
 
+/// A detailed query to server list.
+///
+/// Is constructed from a `ServerQuery`.
+#[derive(Clone, Debug)]
+pub struct DetailedServerQuery {
+    inner: ServerQuery,
+}
+
 /// Structure representing a single server.
 #[derive(Clone, Debug)]
 pub struct Server {
     session: Rc<Session>,
-    inner: protocol::Server
+    inner: protocol::Server,
+    flavor: protocol::ServerFlavor,
 }
 
 /// Structure representing a summary of a single server.
@@ -97,51 +107,71 @@ pub struct ServerCreationWaiter {
 impl Refresh for Server {
     /// Refresh the server.
     fn refresh(&mut self) -> Result<()> {
-        self.inner = self.session.get_server(&self.inner.id)?;
+        self.inner = self.session.get_server_by_id(&self.inner.id)?;
         Ok(())
     }
 }
 
 impl Server {
-    /// Load a Server object.
-    pub(crate) fn new<Id: AsRef<str>>(session: Rc<Session>, id: Id)
+    /// Create a new Server object.
+    pub(crate) fn new(session: Rc<Session>, inner: protocol::Server)
             -> Result<Server> {
-        let inner = session.get_server(id)?;
+        let flavor = session.get_flavor(&inner.flavor.id)?;
         Ok(Server {
             session: session,
-            inner: inner
+            inner: inner,
+            flavor: protocol::ServerFlavor {
+                ephemeral_size: flavor.ephemeral,
+                extra_specs: flavor.extra_specs,
+                original_name: flavor.name,
+                ram_size: flavor.ram,
+                root_size: flavor.disk,
+                swap_size: flavor.swap,
+                vcpu_count: flavor.vcpus,
+            },
         })
     }
 
-    /// Get the IPv4 address.
-    ///
-    /// If not None, this address should be used to access the server instead
-    /// of one from the `addresses` method.
-    pub fn access_ipv4(&self) -> Option<Ipv4Addr> {
-        self.inner.accessIPv4
+    /// Load a Server object.
+    pub(crate) fn load<Id: AsRef<str>>(session: Rc<Session>, id: Id)
+            -> Result<Server> {
+        let inner = session.get_server(id)?;
+        Server::new(session, inner)
     }
 
-    /// Get the IPv6 address.
-    ///
-    /// If not None, this address should be used to access the server instead
-    /// of one from the `addresses` method.
-    pub fn access_ipv6(&self) -> Option<Ipv6Addr> {
-        self.inner.accessIPv6
+    transparent_property! {
+        #[doc = "IPv4 address to access the server (if provided)."]
+        access_ipv4: Option<Ipv4Addr>
     }
 
-    /// Get a reference to associated addresses.
-    pub fn addresses(&self) -> &HashMap<String, Vec<protocol::ServerAddress>> {
-        &self.inner.addresses
+    transparent_property! {
+        #[doc = "IPv6 address to access the server (if provided)."]
+        access_ipv6: Option<Ipv6Addr>
     }
 
-    /// Get a reference to the availability zone.
-    pub fn availability_zone(&self) -> &String {
-        &self.inner.availability_zone
+    transparent_property! {
+        #[doc = "Addresses (floating and fixed) associated with the server."]
+        addresses: ref HashMap<String, Vec<protocol::ServerAddress>>
     }
 
-    /// Get the creation date and time.
-    pub fn created_at(&self) -> DateTime<FixedOffset> {
-        self.inner.created
+    transparent_property! {
+        #[doc = "Availability zone."]
+        availability_zone: ref String
+    }
+
+    transparent_property! {
+        #[doc = "Creation date and time."]
+        created_at: DateTime<FixedOffset>
+    }
+
+    transparent_property! {
+        #[doc = "Server description."]
+        description: ref Option<String>
+    }
+
+    /// Flavor information used to create this server.
+    pub fn flavor(&self) -> &protocol::ServerFlavor {
+        &self.flavor
     }
 
     /// Find a floating IP, if it exists.
@@ -154,9 +184,33 @@ impl Server {
             .map(|a| a.addr).next()
     }
 
-    /// Get a reference to server unique ID.
-    pub fn id(&self) -> &String {
-        &self.inner.id
+    transparent_property! {
+        #[doc = "Whether the server was created with a config drive."]
+        has_config_drive: bool
+    }
+
+    /// Whether the server has an image.
+    ///
+    /// May return `false` if the server was created from a volume.
+    pub fn has_image(&self) -> bool {
+        self.inner.image.is_some()
+    }
+
+    transparent_property! {
+        #[doc = "Server unique ID."]
+        id: ref String
+    }
+
+    /// Fetch the associated image.
+    ///
+    /// Fails with `ResourceNotFound` if the server does not have an image.
+    #[cfg(feature = "image")]
+    pub fn image(&self) -> Result<Image> {
+        match self.inner.image {
+            Some(ref image) => Image::new(self.session.clone(), &image.id),
+            None => Err(Error::new(ErrorKind::ResourceNotFound,
+                                   "No image associated with server"))
+        }
     }
 
     /// Get a reference to the image.
@@ -169,36 +223,43 @@ impl Server {
         }
     }
 
-    /// Whether the server has an image.
-    ///
-    /// May return `false` if the server was created from a volume.
-    pub fn has_image(&self) -> bool {
-        self.inner.image.is_some()
+    /// Fetch the key pair used for the server.
+    pub fn key_pair(&self) -> Result<KeyPair> {
+        match self.inner.key_pair_name {
+            Some(ref key_pair) => KeyPair::new(self.session.clone(), key_pair),
+            None => Err(Error::new(ErrorKind::ResourceNotFound,
+                                   "No key pair associated with server"))
+        }
     }
 
-    /// Get a reference to server name.
-    pub fn name(&self) -> &String {
-        &self.inner.name
+    transparent_property! {
+        #[doc = "Name of a key pair used with this server (if any)."]
+        key_pair_name: ref Option<String>
     }
 
-    /// Get a reference to associated metadata.
-    pub fn metadata(&self) -> &HashMap<String, String> {
-        &self.inner.metadata
+    transparent_property! {
+        #[doc = "Server name."]
+        name: ref String
     }
 
-    /// Get server power state.
-    pub fn power_state(&self) -> protocol::ServerPowerState {
-        self.inner.power_state
+    transparent_property! {
+        #[doc = "Metadata associated with the server."]
+        metadata: ref HashMap<String, String>
     }
 
-    /// Get server status.
-    pub fn status(&self) -> protocol::ServerStatus {
-        self.inner.status
+    transparent_property! {
+        #[doc = "Server power state."]
+        power_state: protocol::ServerPowerState
     }
 
-    /// Get the last update date and time.
-    pub fn updated_at(&self) -> DateTime<FixedOffset> {
-        self.inner.updated
+    transparent_property! {
+        #[doc = "Server status."]
+        status: protocol::ServerStatus
+    }
+
+    transparent_property! {
+        #[doc = "Last update date and time."]
+        updated_at: DateTime<FixedOffset>
     }
 
     /// Delete the server.
@@ -294,7 +355,7 @@ impl ServerSummary {
 
     /// Get details.
     pub fn details(&self) -> Result<Server> {
-        Server::new(self.session.clone(), &self.inner.id)
+        Server::load(self.session.clone(), &self.inner.id)
     }
 
     /// Delete the server.
@@ -411,6 +472,13 @@ impl ServerQuery {
         self
     }
 
+    /// Convert this query into a detailed query.
+    pub fn detailed(self) -> DetailedServerQuery {
+        DetailedServerQuery {
+            inner: self
+        }
+    }
+
     /// Convert this query into an iterator executing the request.
     ///
     /// This iterator yields only `ServerSummary` objects, containing
@@ -420,9 +488,9 @@ impl ServerQuery {
     /// call returning a `Result`.
     ///
     /// Note that no requests are done until you start iterating.
-    pub fn into_iter(self) -> ResourceIterator<ServerSummary> {
+    pub fn into_iter(self) -> ResourceIterator<ServerQuery> {
         debug!("Fetching servers with {:?}", self.query);
-        ResourceIterator::new(self.session, self.query)
+        ResourceIterator::new(self)
     }
 
     /// Convert this query into an iterator executing the request.
@@ -434,9 +502,9 @@ impl ServerQuery {
     /// call returning a `Result`.
     ///
     /// Note that no requests are done until you start iterating.
-    pub fn into_iter_detailed(self) -> ResourceIterator<Server> {
-        debug!("Fetching server details with {:?}", self.query);
-        ResourceIterator::new(self.session, self.query)
+    #[deprecated(since = "0.2.0", note = "Use .detailed().into_iter()")]
+    pub fn into_iter_detailed(self) -> ResourceIterator<DetailedServerQuery> {
+        self.detailed().into_iter()
     }
 
     /// Execute this request and return all results.
@@ -462,16 +530,92 @@ impl ServerQuery {
     }
 }
 
+impl ResourceQuery for ServerQuery {
+    type Item = ServerSummary;
+
+    const DEFAULT_LIMIT: usize = 100;
+
+    fn can_paginate(&self) -> Result<bool> {
+        Ok(self.can_paginate)
+    }
+
+    fn extract_marker(&self, resource: &Self::Item) -> String {
+        resource.id().clone()
+    }
+
+    fn fetch_chunk(&self, limit: Option<usize>, marker: Option<String>)
+            -> Result<Vec<Self::Item>> {
+        let query = self.query.with_marker_and_limit(limit, marker);
+        Ok(self.session.list_servers(&query)?.into_iter().map(|srv| ServerSummary {
+            session: self.session.clone(),
+            inner: srv
+        }).collect())
+    }
+}
+
+impl DetailedServerQuery {
+    /// Convert this query into an iterator executing the request.
+    ///
+    /// This iterator yields full `Server` objects.
+    ///
+    /// Returns a `FallibleIterator`, which is an iterator with each `next`
+    /// call returning a `Result`.
+    ///
+    /// Note that no requests are done until you start iterating.
+    pub fn into_iter(self) -> ResourceIterator<DetailedServerQuery> {
+        debug!("Fetching server details with {:?}", self.inner.query);
+        ResourceIterator::new(self)
+    }
+}
+
+impl ResourceQuery for DetailedServerQuery {
+    type Item = Server;
+
+    const DEFAULT_LIMIT: usize = 50;
+
+    fn can_paginate(&self) -> Result<bool> {
+        Ok(self.inner.can_paginate)
+    }
+
+    fn extract_marker(&self, resource: &Self::Item) -> String {
+        resource.id().clone()
+    }
+
+    fn fetch_chunk(&self, limit: Option<usize>, marker: Option<String>)
+            -> Result<Vec<Self::Item>> {
+        let query = self.inner.query.with_marker_and_limit(limit, marker);
+        let servers = self.inner.session.list_servers_detail(&query)?;
+        let mut result = Vec::with_capacity(servers.len());
+        for srv in servers {
+            result.push(Server::new(self.inner.session.clone(), srv)?);
+        }
+        Ok(result)
+    }
+}
+
+impl From<DetailedServerQuery> for ServerQuery {
+    fn from(value: DetailedServerQuery) -> ServerQuery {
+        value.inner
+    }
+}
+
+impl From<ServerQuery> for DetailedServerQuery {
+    fn from(value: ServerQuery) -> DetailedServerQuery {
+        value.detailed()
+    }
+}
+
 fn convert_networks(session: &Session, networks: Vec<ServerNIC>)
         -> Result<Vec<protocol::ServerNetwork>> {
     let mut result = Vec::with_capacity(networks.len());
     for item in networks {
         result.push(match item {
             ServerNIC::FromNetwork(n) => protocol::ServerNetwork::Network {
-                uuid: n.into_verified(session)?
+                uuid: n.into_verified(session)?.into()
+            },
+            ServerNIC::WithPort(p) => protocol::ServerNetwork::Port {
+                port: p.into_verified(session)?.into()
             },
-            ServerNIC::WithPort(p) =>
-                protocol::ServerNetwork::Port { port: p.into() },
             ServerNIC::WithFixedIp(ip) =>
                 protocol::ServerNetwork::FixedIp{ fixed_ip: ip }
         });
@@ -497,13 +641,13 @@ impl NewServer {
     /// Request creation of the server.
     pub fn create(self) -> Result<ServerCreationWaiter> {
         let request = protocol::ServerCreate {
-            flavorRef: self.flavor.into_verified(&self.session)?,
+            flavorRef: self.flavor.into_verified(&self.session)?.into(),
             imageRef: match self.image {
-                Some(img) => Some(img.into_verified(&self.session)?),
+                Some(img) => Some(img.into_verified(&self.session)?.into()),
                 None => None
             },
             key_name: match self.keypair {
-                Some(item) => Some(item.into_verified(&self.session)?),
+                Some(item) => Some(item.into_verified(&self.session)?.into()),
                 None => None
             },
             metadata: self.metadata,
@@ -513,7 +657,7 @@ impl NewServer {
 
         let server_ref = self.session.create_server(request)?;
         Ok(ServerCreationWaiter {
-            server: Server::new(self.session, server_ref.id)?
+            server: Server::load(self.session, server_ref.id)?
         })
     }
 
@@ -637,50 +781,26 @@ impl WaiterCurrentState<Server> for ServerCreationWaiter {
     }
 }
 
-impl ResourceId for ServerSummary {
-    fn resource_id(&self) -> String {
-        self.id().clone()
-    }
-}
-
-impl ListResources for ServerSummary {
-    const DEFAULT_LIMIT: usize = 50;
-
-    fn list_resources<Q: Serialize + Debug>(session: Rc<Session>, query: Q)
-            -> Result<Vec<ServerSummary>> {
-        Ok(session.list_servers(&query)?.into_iter().map(|srv| ServerSummary {
-            session: session.clone(),
-            inner: srv
-        }).collect())
-    }
-}
-
-impl ResourceId for Server {
-    fn resource_id(&self) -> String {
-        self.id().clone()
-    }
-}
-
-impl ListResources for Server {
-    const DEFAULT_LIMIT: usize = 50;
-
-    fn list_resources<Q: Serialize + Debug>(session: Rc<Session>, query: Q)
-            -> Result<Vec<Server>> {
-        Ok(session.list_servers_detail(&query)?.into_iter().map(|srv| Server {
-            session: session.clone(),
-            inner: srv
-        }).collect())
-    }
-}
-
 impl IntoFallibleIterator for ServerQuery {
     type Item = ServerSummary;
 
     type Error = Error;
 
-    type IntoIter = ResourceIterator<ServerSummary>;
+    type IntoIter = ResourceIterator<ServerQuery>;
 
-    fn into_fallible_iterator(self) -> ResourceIterator<ServerSummary> {
+    fn into_fallible_iterator(self) -> Self::IntoIter {
+        self.into_iter()
+    }
+}
+
+impl IntoFallibleIterator for DetailedServerQuery {
+    type Item = Server;
+
+    type Error = Error;
+
+    type IntoIter = ResourceIterator<DetailedServerQuery>;
+
+    fn into_fallible_iterator(self) -> Self::IntoIter {
         self.into_iter()
     }
 }
