@@ -16,13 +16,11 @@
 
 use std::fmt::Debug;
 
-use reqwest::{Method, Url};
 use serde::Serialize;
 
 use super::super::Result;
-use super::super::auth::AuthMethod;
-use super::super::common;
-use super::super::session::{Session, ServiceInfo, ServiceType};
+use super::super::common::ApiVersion;
+use super::super::session::{RequestBuilderExt, Session, ServiceType};
 use super::super::utils::{self, ResultExt};
 use super::protocol;
 
@@ -52,17 +50,13 @@ pub trait V2API {
 pub struct V2;
 
 
-const SERVICE_TYPE: &'static str = "image";
-// FIXME(dtantsur): detect versions instead of hardcoding Kilo.
-const VERSION_ID: &'static str = "v2.3";
+const SERVICE_TYPE: &str = "image";
 
 
 impl V2API for Session {
     fn get_image_by_id<S: AsRef<str>>(&self, id: S) -> Result<protocol::Image> {
         trace!("Fetching image {}", id.as_ref());
-        let image = self.request::<V2>(Method::Get,
-                                       &["images", id.as_ref()],
-                                       None)?
+        let image = self.get::<V2>(&["images", id.as_ref()], None)?
            .receive_json::<protocol::Image>()?;
         trace!("Received {:?}", image);
         Ok(image)
@@ -70,7 +64,7 @@ impl V2API for Session {
 
     fn get_image_by_name<S: AsRef<str>>(&self, name: S) -> Result<protocol::Image> {
         trace!("Get image by name {}", name.as_ref());
-        let items = self.request::<V2>(Method::Get, &["images"], None)?
+        let items = self.get::<V2>(&["images"], None)?
             .query(&[("name", name.as_ref())])
             .receive_json::<protocol::ImagesRoot>()?.images;
         let result = utils::one(items, "Image with given name or ID not found",
@@ -82,7 +76,7 @@ impl V2API for Session {
     fn list_images<Q: Serialize + Debug>(&self, query: &Q)
             -> Result<Vec<protocol::Image>> {
         trace!("Listing images with {:?}", query);
-        let result = self.request::<V2>(Method::Get, &["images"], None)?
+        let result = self.get::<V2>(&["images"], None)?
            .query(query).receive_json::<protocol::ImagesRoot>()?.images;
         trace!("Received images: {:?}", result);
         Ok(result)
@@ -95,7 +89,7 @@ impl ServiceType for V2 {
         SERVICE_TYPE
     }
 
-    fn service_info(endpoint: Url, auth: &AuthMethod) -> Result<ServiceInfo> {
-        common::protocol::fetch_service_info(endpoint, auth, SERVICE_TYPE, VERSION_ID)
+    fn major_version_supported(version: ApiVersion) -> bool {
+        version.0 == 2
     }
 }
