@@ -40,6 +40,7 @@ pub struct PortQuery {
     session: Rc<Session>,
     query: Query,
     can_paginate: bool,
+    network: Option<NetworkRef>,
 }
 
 /// A fixed IP address of a port.
@@ -305,6 +306,7 @@ impl PortQuery {
             session: session,
             query: Query::new(),
             can_paginate: true,
+            network: None,
         }
     }
 
@@ -365,19 +367,11 @@ impl PortQuery {
     }
 
     /// Filter by network.
-    ///
-    /// # Warning
-    ///
-    /// Due to architectural limitations, names do not work here.
     pub fn set_network<N: Into<NetworkRef>>(&mut self, value: N) {
-        self.query.push_str("network_id", value.into());
+        self.network = Some(value.into());
     }
 
     /// Filter by network.
-    ///
-    /// # Warning
-    ///
-    /// Due to architectural limitations, names do not work here.
     pub fn with_network<N: Into<NetworkRef>>(mut self, value: N) -> Self {
         self.set_network(value);
         self
@@ -441,6 +435,14 @@ impl ResourceQuery for PortQuery {
         Ok(self.session.list_ports(&query)?.into_iter()
            .map(|item| Port::new(self.session.clone(), item)).collect())
     }
+
+    fn validate(&mut self) -> Result<()> {
+        if let Some(network) = self.network.take() {
+            let verified = network.into_verified(&self.session)?;
+            self.query.push_str("network_id", verified);
+        }
+        Ok(())
+    }
 }
 
 impl NewPort {
