@@ -187,12 +187,41 @@ impl Default for Network {
     }
 }
 
+/// A network.
+#[derive(Debug, Clone, Default, Serialize)]
+pub struct NetworkUpdate {
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub admin_state_up: Option<bool>,
+    #[serde(rename = "router:external", skip_serializing_if = "Option::is_none")]
+    pub external: Option<bool>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub description: Option<String>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub dns_domain: Option<String>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub is_default: Option<bool>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub mtu: Option<u32>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub name: Option<String>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub port_security_enabled: Option<bool>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub shared: Option<bool>,
+}
+
 /// A network.
 #[derive(Debug, Clone, Deserialize, Serialize)]
 pub struct NetworkRoot {
     pub network: Network
 }
 
+/// A network.
+#[derive(Debug, Clone, Serialize)]
+pub struct NetworkUpdateRoot {
+    pub network: NetworkUpdate
+}
+
 /// A list of networks.
 #[derive(Debug, Clone, Deserialize)]
 pub struct NetworksRoot {
@@ -421,12 +450,37 @@ impl Subnet {
     }
 }
 
+/// A subnet.
+#[derive(Debug, Clone, Serialize, Default)]
+pub struct SubnetUpdate {
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub allocation_pools: Option<Vec<AllocationPool>>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub description: Option<String>,
+    #[serde(rename = "enable_dhcp", skip_serializing_if = "Option::is_none")]
+    pub dhcp_enabled: Option<bool>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub dns_nameservers: Option<Vec<String>>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub gateway_ip: Option<net::IpAddr>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub host_routes: Option<Vec<HostRoute>>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub name: Option<String>,
+}
+
 /// A subnet.
 #[derive(Debug, Clone, Deserialize, Serialize)]
 pub struct SubnetRoot {
     pub subnet: Subnet
 }
 
+/// A subnet.
+#[derive(Debug, Clone, Serialize)]
+pub struct SubnetUpdateRoot {
+    pub subnet: SubnetUpdate
+}
+
 /// A list of subnets.
 #[derive(Debug, Clone, Deserialize)]
 pub struct SubnetsRoot {
